@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -16,8 +16,9 @@
 #include "Item.h"
 #include "PWSprefs.h"
 #include "PWPolicy.h"
-#include "os/UUID.h"
+#include "../os/UUID.h"
 #include "StringX.h"
+#include "TotpCore.h"
 
 #include <time.h> // for time_t
 #include <bitset>
@@ -84,12 +85,49 @@ public:
   // Convenience: Get the untranslated (English) name of a FieldType
   static stringT EngFieldName(FieldType ft);
 
+  // Return XML-compliant element name.
+  // Note: Not all XML elements use this immediately. Generally for use with
+  // newly added fields or those where appropriate refactoring has taken place.
+  static std::string GetXmlFieldName(FieldType ft);
+  static std::wstring GetXmlFieldNameW(FieldType ft);
+
+  // Get the field name used in the user interface (i.e., for Advanced XML
+  // export field selection, Display Filters, etc.).
+  static stringT GetUserInterfaceFieldName(FieldType ft);
+
   //Data retrieval
   StringX GetName() const {return GetField(NAME);} // V17 - deprecated: replaced by GetTitle & GetUser
   StringX GetTitle() const {return GetField(TITLE);} // V20
   StringX GetUser() const  {return GetField(USER);}  // V20
-  StringX GetPassword() const {return GetField(PASSWORD);}
-  size_t GetPasswordLength() const {return GetField(PASSWORD).length();}
+  StringX GetPassword() const { return GetField(PASSWORD); }
+  size_t GetPasswordLength() const { return GetField(PASSWORD).length(); }
+
+  //
+  // TOTP-related fields:
+  //
+  bool IsTotpActive() const { return GetTwoFactorKeyLength() != 0; }
+  StringX GetTwoFactorKey() const { return GetField(TWOFACTORKEY); }
+  size_t GetTwoFactorKeyLength() const { return GetField(TWOFACTORKEY).length(); }
+
+  uint8_t GetTotpConfigAsByte() const { return GetFieldAsByte(TOTPCONFIG, PWSTotp::TOTP_CONFIG_ALGORITHM_DEFAULT); }
+  StringX GetTotpConfig() const { return IntegralToStringX(GetTotpConfigAsByte()); }
+  bool IsTotpConfigDefault() const { return GetTotpConfigAsByte() == PWSTotp::TOTP_CONFIG_ALGORITHM_DEFAULT; }
+  uint8_t GetTotpAlgorithmAsByte() const { return GetTotpConfigAsByte() & PWSTotp::TOTP_CONFIG_ALGORITHM_MASK; }
+
+  uint8_t GetTotpLengthAsByte() const { return GetFieldAsByte(TOTPLENGTH, PWSTotp::TOTP_DEFAULT_AUTH_CODE_LENGTH); }
+  StringX GetTotpLength() const { return IntegralToStringX(GetTotpLengthAsByte()); }
+  bool IsTotpLengthDefault() const { return GetTotpLengthAsByte() == PWSTotp::TOTP_DEFAULT_AUTH_CODE_LENGTH; }
+
+  uint8_t GetTotpTimeStepSecondsAsByte() const { return GetFieldAsByte(TOTPTIMESTEP, PWSTotp::TOTP_DEFAULT_TIME_STEP_SECONDS); }
+  StringX GetTotpTimeStepSeconds() const { return IntegralToStringX(GetFieldAsByte(TOTPTIMESTEP, PWSTotp::TOTP_DEFAULT_TIME_STEP_SECONDS)); }
+  bool IsTotpTimeStepSecondsDefault() const { return GetTotpTimeStepSecondsAsByte() == PWSTotp::TOTP_DEFAULT_TIME_STEP_SECONDS; }
+
+  time_t GetTotpStartTimeAsTimeT() const { time_t t;  CItem::GetTime(TOTPSTARTTIME, t); return t; }
+  StringX GetTotpStartTime() const { return GetTime(TOTPSTARTTIME, PWSUtil::TMC_ASC_UNKNOWN, true, true); }
+  bool IsTotpStartTimeDefault() const { return GetTotpStartTimeAsTimeT() == 0; }
+  StringX GetTotpAuthCode(time_t* pBasisTimeNow = nullptr, double* pRatioExpired = nullptr) const;
+
+
   StringX GetNotes(TCHAR delimiter = 0) const;
   void GetUUID(uuid_array_t &, FieldType ft = END) const; // V20
   const pws_os::CUUID GetUUID(FieldType ft = END) const; // V20 - see comment in .cpp re return type
@@ -170,6 +208,13 @@ public:
   void SetUser(const StringX &user) {CItem::SetField(USER, user);} // V20
   void SetPassword(const StringX &password) {CItem::SetField(PASSWORD, password);}
   void UpdatePassword(const StringX &password); // use when password changed!
+  void ClearTwoFactorKey() {ClearField(CItemData::TWOFACTORKEY);}
+  void SetTwoFactorKey(const StringX& value) { CItem::SetField(TWOFACTORKEY, value); }
+  bool SetTotpConfig(const StringX& value) { return SetFieldAsByte(TOTPCONFIG, value.c_str()); }
+  void SetTotpStartTime(time_t t) { CItem::SetTime(TOTPSTARTTIME, t); }
+  bool SetTotpStartTime(const StringX& value) { return SetTime(TOTPSTARTTIME, value.c_str(), true); }
+  bool SetTotpTimeStep(const StringX& value) { return SetFieldAsByte(TOTPTIMESTEP, value.c_str()); }
+  bool SetTotpLength(const StringX& value) { return SetFieldAsByte(TOTPLENGTH, value.c_str()); }
   void SetNotes(const StringX &notes, TCHAR delimiter = 0);
   void SetUUID(const pws_os::CUUID &uuid, FieldType ft = CItemData::UUID);
   void SetBaseUUID(const pws_os::CUUID &uuid) {SetUUID(uuid, CItemData::BASEUUID);}
@@ -308,6 +353,29 @@ public:
   bool IsURLEmail(const CItemData *pbci) const
   { return GetEffectiveFieldValue(URL, pbci).find(_T("mailto:")) != StringX::npos; }
 
+  // V3 attachment
+  bool IsAttTitleSet() const            { return IsFieldSet(DATA_ATT_TITLE);     }
+  bool IsAttMediaTypeSet() const        { return IsFieldSet(DATA_ATT_MEDIATYPE); }
+  bool IsAttFileNameSet() const         { return IsFieldSet(DATA_ATT_FILENAME);  }
+  bool IsAttModificationTimeSet() const { return IsFieldSet(DATA_ATT_MTIME);     }
+  bool IsAttContentSet() const          { return IsFieldSet(DATA_ATT_CONTENT);   }
+
+  StringX GetAttTitle() const           { return GetField(DATA_ATT_TITLE);     }
+  StringX GetAttMediaType() const       { return GetField(DATA_ATT_MEDIATYPE); }
+  StringX GetAttFileName() const        { return GetField(DATA_ATT_FILENAME);  }
+  time_t GetAttModificationTime(time_t &t) const { CItem::GetTime(DATA_ATT_MTIME, t); return t; }
+  size_t GetAttContentLength() const;
+  std::vector<unsigned char> GetAttContent() const;
+
+  void SetAttTitle(const StringX &title)         { CItem::SetField(DATA_ATT_TITLE, title); }
+  void SetAttMediaType(const StringX &mediaType) { CItem::SetField(DATA_ATT_MEDIATYPE, mediaType); }
+  void SetAttFileName(const StringX &fileName)   { CItem::SetField(DATA_ATT_FILENAME, fileName); }
+  void SetAttModificationTime(time_t t)          { CItem::SetTime(DATA_ATT_MTIME, t); }
+  void SetAttContent(const unsigned char *content, size_t clen) { CItem::SetField(DATA_ATT_CONTENT, content, clen); }
+
+  bool HasAttachment() const            { return IsAttMediaTypeSet();            }
+  void ClearAttachment();
+
 private:
   EntryType m_entrytype;
   EntryStatus m_entrystatus;
@@ -315,12 +383,14 @@ private:
   // move from pre-2.0 name to post-2.0 title+user
   void SplitName(const StringX &name,
                  StringX &title, StringX &username);
-  StringX GetTime(int whichtime, PWSUtil::TMC result_format) const; // V30
+  StringX GetTime(int whichtime, PWSUtil::TMC result_format, bool convert_epoch = false, bool utc_time = false) const; // V30
   void SetTime(const int whichtime); // V30
-  bool SetTime(const int whichtime, const stringT &time_str); // V30
+  bool SetTime(const int whichtime, const stringT &time_str, bool utc_time = false); // V30
 
   // Laziness is a Virtue:
-  bool SetField(unsigned char type, const unsigned char *data, size_t len);
+  bool SetField(unsigned char ft_byte, const unsigned char *data, size_t len);
+  bool SetField(CItem::FieldType ft, const unsigned char* data, size_t len);
+  bool SetFieldAsByte(CItem::FieldType type, const stringT& byte_str, bool strict = true);
 
   // for V3 Alias or Shortcut, the base UUID is encoded in password
   void ParseSpecialPasswords();
@@ -341,6 +411,8 @@ inline bool CItemData::IsTextField(unsigned char t)
     t == RESERVED   || t == DCA    || t == SHIFTDCA || t == PROTECTED ||
     t == KBSHORTCUT || t == ATTREF || t == BASEUUID || t == ALIASUUID ||
     t == SHORTCUTUUID ||
+    t == TOTPCONFIG || t == TOTPLENGTH || t == TOTPSTARTTIME || t == TOTPTIMESTEP ||
+    t == DATA_ATT_MTIME || t == DATA_ATT_CONTENT ||
     t >= LAST_DATA);
 }
 #endif /* __ITEMDATA_H */
