@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -54,9 +54,10 @@ using pws_os::CUUID;
  *         V3.29Y          0x030C
  *         V3.30           0x030D
  *         V3.47           0x030E
+ *         V3.68           0x030F (new fields and MIN_HASH_ITERATIONS updated)
 */
 
-const short VersionNum = 0x030E;
+const short VersionNum = 0x030F;
 
 static unsigned char TERMINAL_BLOCK[TwoFish::BLOCKSIZE] = {
   'P', 'W', 'S', '3', '-', 'E', 'O', 'F',
@@ -123,12 +124,12 @@ int PWSfileV3::Close()
     size_t fret;
     fret = fwrite(TERMINAL_BLOCK, sizeof(TERMINAL_BLOCK), 1, m_fd);
     if (fret != 1) {
-      PWSfile::Close();
+      (void)PWSfile::Close();
       return FAILURE;
     }
     fret = fwrite(digest, sizeof(digest), 1, m_fd);
     if (fret != 1) {
-      PWSfile::Close();
+      (void)PWSfile::Close();
       return FAILURE;
     }
     return PWSfile::Close();
@@ -136,11 +137,11 @@ int PWSfileV3::Close()
     // We're here *after* TERMINAL_BLOCK has been read
     // and detected (by _readcbc) - just read hmac & verify
     unsigned char d[SHA256::HASHLEN];
-    (void) fread(d, sizeof(d), 1, m_fd);
-    if (memcmp(d, digest, SHA256::HASHLEN) == 0)
+    if (fread(d, sizeof(d), 1, m_fd) == 1 &&
+        memcmp(d, digest, SHA256::HASHLEN) == 0)
       return PWSfile::Close();
     else {
-      PWSfile::Close();
+      (void)PWSfile::Close();
       return BAD_DIGEST;
     }
   }
@@ -221,17 +222,21 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
 
   fseek(fd, sizeof(V3TAG), SEEK_SET); // skip over tag
   unsigned char salt[PWSaltLength];
-  (void) fread(salt, 1, sizeof(salt), fd);
+  if (fread(salt, 1, sizeof(salt), fd) != sizeof(salt)) {
+    retval = READ_FAIL;
+    goto err;
+  }
 
   unsigned char Nb[sizeof(uint32)];
-  (void) fread(Nb, 1, sizeof(Nb), fd);
+  if (fread(Nb, 1, sizeof(Nb), fd) != sizeof(Nb)) {
+    retval = READ_FAIL;
+    goto err;
+  }
   { // block to shut up compiler warning w.r.t. goto
     const uint32 N = getInt32(Nb);
 
-    ASSERT(N >= MIN_HASH_ITERATIONS);
     if (N < MIN_HASH_ITERATIONS) {
-      retval = FAILURE;
-      goto err;
+      PWSTRACE(L"File's ITER value %d is below current minimum %d. It will be updated when file is saved", N, MIN_HASH_ITERATIONS);
     }
 
     if (nITER != nullptr)
@@ -243,7 +248,10 @@ int PWSfileV3::CheckPasskey(const StringX &filename,
   H.Update(usedPtag, SHA256::HASHLEN);
   H.Final(HPtag);
   unsigned char readHPtag[SHA256::HASHLEN];
-  (void) fread(readHPtag, 1, sizeof(readHPtag), fd);
+  if (fread(readHPtag, 1, sizeof(readHPtag), fd) != sizeof(readHPtag)) {
+    retval = READ_FAIL;
+    goto err;
+  }
   if (memcmp(readHPtag, HPtag, sizeof(readHPtag)) != 0) {
     retval = WRONG_PASSWORD;
     goto err;
@@ -321,7 +329,6 @@ void PWSfileV3::StretchKey(const unsigned char *salt, unsigned long saltLen,
   trashMemory(pstr, passLen);
   delete[] pstr;
 
-  ASSERT(N >= MIN_HASH_ITERATIONS); // minimal value we're willing to use
   for (unsigned int i = 0; i < N; i++) {
     SHA256 H;
     // The 2nd param in next line was sizeof(X) in Beta-1
@@ -602,7 +609,10 @@ int PWSfileV3::ReadHeader()
 
   unsigned char B1B2[sizeof(m_key)];
   ASSERT(sizeof(B1B2) == 32); // Generalize later
-  (void) fread(B1B2, 1, sizeof(B1B2), m_fd);
+  if (fread(B1B2, 1, sizeof(B1B2), m_fd) != sizeof(B1B2)) {
+    Close();
+    return READ_FAIL;
+  }
   TwoFish TF(Ptag, sizeof(Ptag));
   TF.Decrypt(B1B2, m_key);
   TF.Decrypt(B1B2 + 16, m_key + 16);
@@ -610,13 +620,19 @@ int PWSfileV3::ReadHeader()
   unsigned char L[32]; // for HMAC
   unsigned char B3B4[sizeof(L)];
   ASSERT(sizeof(B3B4) == 32); // Generalize later
-  (void) fread(B3B4, 1, sizeof(B3B4), m_fd);
+  if (fread(B3B4, 1, sizeof(B3B4), m_fd) != sizeof(B3B4)) {
+    Close();
+    return READ_FAIL;
+  }
   TF.Decrypt(B3B4, L);
   TF.Decrypt(B3B4 + 16, L + 16);
 
   m_hmac.Init(L, sizeof(L));
 
-  (void) fread(m_ipthing, 1, sizeof(m_ipthing), m_fd);
+  if (fread(m_ipthing, 1, sizeof(m_ipthing), m_fd) != sizeof(m_ipthing)) {
+    Close();
+    return READ_FAIL;
+  }
 
   m_fish = new TwoFish(m_key, sizeof(m_key));
 
@@ -983,9 +999,15 @@ int PWSfileV3::ReadHeader()
     FILE *fd = pws_os::FOpen(filename.c_str(), _T("rb"));
 
     ASSERT(fd != nullptr);
+    if (fd == nullptr)
+      return false;
     char tag[sizeof(V3TAG)];
-    fread(tag, 1, sizeof(tag), fd);
+    auto nread = fread(tag, 1, sizeof(tag), fd);
     fclose(fd);
+    if (nread != sizeof(tag)) {
+      v = UNKNOWN_VERSION;
+      return false;
+    }
 
     if (memcmp(tag, V3TAG, sizeof(tag)) == 0) {
       v = V30;
