@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -18,6 +18,7 @@
  */
 
 ////@begin includes
+#include <wx/bookctrl.h>
 #include <wx/combobox.h>
 #include <wx/propdlg.h>
 #include <wx/valgen.h>
@@ -33,6 +34,9 @@
 #include "core/ItemData.h"
 #include "core/PWScore.h"
 
+#include "PasswordSafeFrame.h"
+#include "PWSafeApp.h"
+
 #include "wxUtilities.h"
 
 /*!
@@ -66,6 +70,11 @@ class wxBoxSizer;
 #define ID_BUTTON_SHOWHIDE 10090
 #define ID_BUTTON_GENERATE 10097
 #define ID_TEXTCTRL_PASSWORD2 10091
+#define ID_STATICTEXT_PASSWORD2 10191
+#define ID_TEXTCTRL_TOTP 11191
+#define ID_BUTTON_SHOWHIDE_TOTP 11192
+#define ID_BUTTON_COPY_TOTP 11193
+#define ID_TIMER_TOTP_COUNTDOWN 11194
 #define ID_TEXTCTRL_URL 10092
 #define ID_GO_BTN 10093
 #define ID_TEXTCTRL_EMAIL 10100
@@ -76,6 +85,8 @@ class wxBoxSizer;
 #define ID_TEXTCTRL_RUN_CMD 10099
 #define ID_COMBOBOX_DBC_ACTION 10101
 #define ID_COMBOBOX_SDBC_ACTION 10000
+#define ID_TEXTCTRL_2FK 10002
+#define ID_BUTTON_SHOWHIDE_2FK 10003
 #define ID_CHECKBOX_KEEP 10102
 #define ID_SPINCTRL_MAX_PW_HIST 10103
 #define ID_GRID_PW_HIST 10104
@@ -146,7 +157,7 @@ public:
                       long style = SYMBOL_ADDEDITPROPSHEETDLG_STYLE);
 
   /// Destructor
-  ~AddEditPropSheetDlg() = default;
+  ~AddEditPropSheetDlg();
 protected:
   /// Constructor
   AddEditPropSheetDlg(wxWindow *parent, PWScore &core,
@@ -156,9 +167,21 @@ protected:
 
   ////@begin AddEditPropSheetDlg event handler declarations
 
+  /// wxEVT_TEXT event handler for ID_TEXTCTRL_PASSWORD and ID_TEXTCTRL_PASSWORD2
+  void OnPasswordChanged(wxCommandEvent &event);
+
   /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_SHOWHIDE
   void OnShowHideClick(wxCommandEvent &event);
 
+  /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_SHOWHIDE_2FK
+  void OnShowHide2FKClick(wxCommandEvent &event);
+
+  /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_SHOWHIDE_TOTP
+  void OnShowHideTotpClick(wxCommandEvent &event);
+
+  /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_COPY_TOTP
+  void OnCopyAuthCodeClick(wxCommandEvent &event);
+
   /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_GENERATE
   void OnGenerateButtonClick(wxCommandEvent &event);
   
@@ -235,6 +258,12 @@ protected:
 
   /// wxEVT_SPINCTRL event handler for ID_SPINCTRL5, ID_SPINCTRL6, ID_SPINCTRL7, ID_SPINCTRL8
   void OnAtLeastPasswordChars(wxSpinEvent &event);
+
+  /// wxEVT_TIMER_EVENT event handler for ID_TIMER_TOTP_COUNTDOWN
+  void OnTotpCountdownTimer(wxTimerEvent &event);
+
+  /// wxEVT_NOTEBOOK_PAGE_CHANGING event handler
+  void OnTabChanging(wxBookCtrlEvent& event);
   ////@begin AddEditPropSheetDlg member function declarations
 
   /// Retrieves bitmap resources
@@ -288,8 +317,15 @@ private:
 
   void ItemFieldsToPropSheet();
   void SetupDCAComboBoxes(wxComboBox *pcbox, short &iDCA, bool isShift);
-  void UpdateExpTimes();        // entry -> controls
+  void InitializeExpTimes();        // entry -> controls
   void SetXTime(wxObject *src); // sync controls + controls -> entry
+
+  // Today returns time part == 0
+  wxDateTime TodayPlusInterval(const int interval) const { return wxDateTime::Today().Add(wxDateSpan(0, 0, 0, interval)); };
+  wxDateTime NormalizeExpDate(const wxDateTime &xdt) const { return xdt.GetDateOnly(); }
+  int IntervalFromDate(const wxDateTime &xdt) const { return xdt.Subtract(wxDateTime::Today()).GetDays(); }
+  wxString makeExpiryString();
+
   void UpdatePWPolicyControls(const PWPolicy &pwp);
   void EnablePWPolicyControls(bool enable);
   PWPolicy GetPWPolicyFromUI() const;
@@ -297,18 +333,36 @@ private:
   bool CheckPWPolicyFromUI();
   void ShowPWPSpinners(bool show);
   void EnableNonHexCBs(bool enable);
-  void UpdatePasswordTextCtrl(wxTextCtrl* &textCtrl, const wxString value, wxTextCtrl* before, const int id = ID_TEXTCTRL_PASSWORD, const int style = 0);
   void ShowPassword();
   void HidePassword();
+  void UpdatePasswordConfirmationIcons(bool show = true);
+  void UpdatePasswordConfirmationAsterisk(bool show = true);
   void ShowAlias();
   void RemoveAlias();
   int GetRequiredPWLength() const;
 
   bool ValidateBasicData();
+  bool ValidateAdditionalData();
   bool ValidatePasswordPolicy();
   bool IsGroupUsernameTitleCombinationUnique();
   bool SyncAndQueryCancel(bool showDialog);
-  
+
+  void StartTotp();
+  void StopTotp();
+  void UpdateTotp();
+  void ShowTotp();
+  void HideTotp();
+  void ShowTwoFactorKey();
+  void HideTwoFactorKey();
+  void ApplyTwoFactorKey(CItemData& item);
+  void EnableAuthenticationCodeControls();
+  void DisableAuthenticationCodeControls();
+  const PasswordSafeFrame* GetPwSafe() const { return wxGetApp().GetPasswordSafeFrame(); }
+  bool HasItemTwoFactorKey() const { return GetPwSafe()->HasItemTwoFactorKey(&m_ItemTotp); };
+  bool IsItemNormalOrBase() const { return GetPwSafe()->IsItemNormalOrBase(&m_ItemTotp); }
+  const CItemData *GetTotpItem() const { return GetPwSafe()->GetTotpItem(&m_ItemTotp); };
+  int GetTotpCountdownInterval() const { return GetPwSafe()->GetTotpCountdownInterval(); }
+
   enum Changes : uint32_t {
     None = 0,
     Group = 1u,
@@ -328,6 +382,15 @@ private:
     Password = 1u << 14,
     Policy = 1u << 15,
     Attachment = 1u << 16,
+    XTimeNever = 1u << 17,
+    TwoFactorKey = 1u << 18,
+  };
+
+  enum AliasChanges {
+    NoChange,
+    Alias2Normal,
+    AliasRebased,
+    Normal2Alias,
   };
   
   uint32_t GetChanges() const;
@@ -347,9 +410,15 @@ private:
   wxTextCtrl *m_BasicUsernameTextCtrl = nullptr;
   wxTextCtrl *m_BasicPasswordTextCtrl = nullptr;
   wxStaticText *m_BasicPasswordTextLabel = nullptr;
-  wxButton *m_BasicShowHideCtrl = nullptr;
+  wxStaticBitmap *m_BasicPasswordBitmap = nullptr;
+  wxBitmapButton *m_BasicShowHideCtrl = nullptr;
   wxTextCtrl *m_BasicPasswordConfirmationTextCtrl = nullptr;
   wxStaticText *m_BasicPasswordConfirmationTextLabel = nullptr;
+  wxStaticBitmap *m_BasicPasswordConfirmationBitmap = nullptr;
+  wxStaticText *m_BasicTotpTextLabel = nullptr;
+  wxTextCtrl *m_BasicTotpTextCtrl = nullptr;
+  wxBitmapButton *m_BasicShowHideTotpCtrl = nullptr;
+  wxButton *m_BasicTotpButton = nullptr;
   wxTextCtrl *m_BasicUrlTextCtrl = nullptr;
   wxTextCtrl *m_BasicEmailTextCtrl = nullptr;
   wxTextCtrl *m_BasicNotesTextCtrl = nullptr;
@@ -359,17 +428,25 @@ private:
   wxString m_Url;
   wxString m_Email;
   wxString m_Notes;
-  bool m_IsNotesHidden;
+  bool m_IsNotesHidden = true;
   StringX m_Password;
-  bool m_IsPasswordHidden;
+  bool m_IsPasswordHidden = true;
+  bool m_IsTotpHidden = true;
+  bool m_UpdateTotpInClipboard = false;
+
+  AliasChanges m_AliasChange = AliasChanges::NoChange;
 
   // Tab: "Additional"
   wxPanel *m_AdditionalPanel = nullptr;
   wxComboBox *m_AdditionalDoubleClickActionCtrl = nullptr;
   wxComboBox *m_AdditionalShiftDoubleClickActionCtrl = nullptr;
+  wxBoxSizer *m_AdditionalHBoxSizerTwoFactorKey = nullptr;
+  wxTextCtrl *m_AdditionalTwoFactorKeyCtrl = nullptr;
+  wxBitmapButton *m_AdditionalShowHideCtrl = nullptr;
   wxSpinCtrl *m_AdditionalMaxPasswordHistoryCtrl = nullptr;
   wxGrid *m_AdditionalPasswordHistoryGrid = nullptr;
 
+  bool m_IsTwoFactorKeyHidden = true;
   wxString m_Autotype;
   wxString m_RunCommand;
   wxString m_PasswordHistory; // String as stored in CItemData
@@ -385,16 +462,20 @@ private:
   wxSpinCtrl *m_DatesTimesExpiryTimeCtrl = nullptr;
   wxCheckBox *m_DatesTimesRecurringExpiryCtrl = nullptr;
   wxRadioButton *m_DatesTimesNeverExpireCtrl = nullptr;
+  wxStaticText *m_DatesTimesCurrentCtrl = nullptr;
+  wxStaticText *m_DatesTimesStaticTextDays = nullptr;
 
   wxString m_RMTime; // Any field modification time
   wxString m_AccessTime;
   wxString m_CreationTime;
-  wxString m_CurrentExpirationTime;
+  wxString m_OriginalExpirationStr;
   wxString m_ModificationTime;
+  wxRadioButton *m_OriginalButton;
   bool m_Recurring;
-  wxString m_ExpirationTime;
+  bool m_OriginalRecurring;
+  time_t m_OriginalDayttt;
+  bool m_FirstInClick;
   int m_ExpirationTimeInterval = 0; // Password expiration interval in days
-  time_t m_tttExpirationTime;   // Password expiration date in time_t
 
   // Tab: "Password Policy"
   wxPanel *m_PasswordPolicyPanel = nullptr;
@@ -459,7 +540,14 @@ private:
 
   SheetType m_Type;
   CItemData m_Item;
+  CItemData m_ItemTotp;
   CItemAtt  m_ItemAttachment;
+
+  wxBitmap bitmapCheckmarkPlaceholder;
+  wxBitmap bitmapCheckmarkGreen;
+  wxBitmap bitmapCheckmarkGray;
+
+  wxTimer *m_TotpTimer = nullptr;
 };
 
 #endif // _ADDEDITPROPSHEETDLG_H_
