@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -46,6 +46,27 @@
 
 #include "wxUtilities.h"
 
+#if defined(__UNIX__) && !defined(__WXOSX__)
+const char *KDEClipboardSecretMarkerValue = "secret";
+
+class KDEClipboardSecretMarkerObject : public wxDataObjectSimple
+{
+public:
+  KDEClipboardSecretMarkerObject() : wxDataObjectSimple(wxDataFormat(wxT("x-kde-passwordManagerHint"))) {}
+
+  virtual bool GetDataHere(void *buf) const wxOVERRIDE
+  {
+    memcpy(buf, KDEClipboardSecretMarkerValue, this->GetDataSize());
+    return true;
+  }
+
+  virtual size_t GetDataSize() const wxOVERRIDE
+  {
+    return strlen(KDEClipboardSecretMarkerValue);
+  }
+};
+#endif
+
 Clipboard *Clipboard::self = nullptr;
 
 /**
@@ -85,7 +106,19 @@ bool Clipboard::SetData(const StringX &data)
 
   bool res = false;
   if (wxTheClipboard->Open()) {
+#if defined(__UNIX__) && !defined(__WXOSX__)
+    // Copying composite object is currently not working as expected on Wayland
+    if (wxUtilities::WhatWindowSystem() == wxUtilities::X11) {
+      wxDataObjectComposite *dataObjectComposite = new wxDataObjectComposite();
+      dataObjectComposite->Add(new wxTextDataObjectEx(data.c_str()), true);
+      dataObjectComposite->Add(new KDEClipboardSecretMarkerObject(), false);
+      res = wxTheClipboard->SetData(dataObjectComposite);
+    } else {
+      res = wxTheClipboard->SetData(new wxTextDataObjectEx(data.c_str()));
+    }
+#else
     res = wxTheClipboard->SetData(new wxTextDataObjectEx(data.c_str()));
+#endif
     wxTheClipboard->Close();
   }
   m_set = true;
@@ -101,6 +134,25 @@ bool Clipboard::SetData(const StringX &data)
   return res;
 }
 
+bool Clipboard::HasData(const StringX &data) const
+{
+  if (wxTheClipboard->Open()) {
+    wxString clipboardText;
+    // Our data is of type text
+    if (
+      wxTheClipboard->IsSupported(wxDF_TEXT) ||
+      wxTheClipboard->IsSupported(wxDF_UNICODETEXT)) 
+    {
+      wxTextDataObject textData;
+      wxTheClipboard->GetData(textData);
+      clipboardText = textData.GetText();
+    }
+    wxTheClipboard->Close();
+    return towxstring(data) == clipboardText;
+  }
+  return false;
+}
+
 /**
  * Clear from clipboard data, that we put there previously
  * @return \c true, if we cleared our data, or stored data don't belong to us
