@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -45,7 +45,7 @@ bool DnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
   
   if (m_pOwner != NULL)
   {
-    if(! m_pOwner->m_core.IsDbOpen()) {
+    if(! m_pOwner->m_core.IsDbFileSet()) {
       if(m_pOwner->Open(filenames[0]) != PWScore::SUCCESS)
         return false;
     }
