@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -23,8 +23,6 @@
 #include <wx/msw/msvcrt.h>
 #endif
 
-#include "core/PWCharPool.h" // for CheckMasterPassword()
-
 ////@begin includes
 #include "ExternalKeyboardButton.h"
 #include "SafeCombinationCtrl.h"
@@ -54,7 +52,7 @@ BEGIN_EVENT_TABLE( SafeCombinationChangeDlg, wxDialog )
 #ifndef NO_YUBI
   EVT_BUTTON( ID_YUBIBTN,                   SafeCombinationChangeDlg::OnYubibtnClick  )
   EVT_BUTTON( ID_YUBIBTN2,                  SafeCombinationChangeDlg::OnYubibtn2Click )
-  EVT_TIMER(  YubiMixin::POLLING_TIMER_ID, SafeCombinationChangeDlg::OnPollingTimer  )
+  EVT_TIMER(  YubiMixin::POLLING_TIMER_ID,  SafeCombinationChangeDlg::OnPollingTimer  )
 #endif
   EVT_BUTTON( wxID_OK,                      SafeCombinationChangeDlg::OnOkClick       )
   EVT_BUTTON( wxID_CANCEL,                  SafeCombinationChangeDlg::OnCancelClick   )
@@ -82,17 +80,13 @@ SafeCombinationChangeDlg::SafeCombinationChangeDlg(wxWindow *parent, PWScore &co
   {
     GetSizer()->SetSizeHints(this);
   }
-  // Allow to resize the dialog in width, only.
-  SetMaxSize(wxSize(wxDefaultCoord, GetMinSize().y));
   Centre();
 ////@end SafeCombinationChangeDlg creation
 #ifndef NO_YUBI
-  m_yubiMixin1.SetupMixin(FindWindow(ID_YUBIBTN), FindWindow(ID_YUBISTATUS));
-  m_yubiMixin1.SetPrompt1(_("Enter old safe combination (if any) and click on top Yubikey button"));
-  m_yubiMixin2.SetupMixin(FindWindow(ID_YUBIBTN2), FindWindow(ID_YUBISTATUS));
-  m_yubiMixin2.SetPrompt1(_("Enter old safe combination (if any) and click on top Yubikey button"));
-  m_pollingTimer = new wxTimer(this, YubiMixin::POLLING_TIMER_ID);
-  m_pollingTimer->Start(2*YubiMixin::POLLING_INTERVAL); // 2 controls
+  m_yubiMixin1.SetupMixin(this, FindWindow(ID_YUBIBTN), FindWindow(ID_YUBISTATUS), YubiMixin::POLLING_TIMER_ID);
+  m_yubiMixin1.SetPrompt1(_("Enter old master password and click on top Yubikey button"));
+  m_yubiMixin2.SetupMixin(this, FindWindow(ID_YUBIBTN2), FindWindow(ID_YUBISTATUS), YubiMixin::POLLING_TIMER_NONE);
+  m_yubiMixin2.SetPrompt1(_("Enter old master password and click on top Yubikey button"));
 #endif
 }
 
@@ -111,9 +105,6 @@ SafeCombinationChangeDlg::~SafeCombinationChangeDlg()
 {
 ////@begin SafeCombinationChangeDlg destruction
 ////@end SafeCombinationChangeDlg destruction
-#ifndef NO_YUBI
-  delete m_pollingTimer;
-#endif
 }
 
 /*!
@@ -122,104 +113,105 @@ SafeCombinationChangeDlg::~SafeCombinationChangeDlg()
 
 void SafeCombinationChangeDlg::CreateControls()
 {
-////@begin SafeCombinationChangeDlg content construction
-  SafeCombinationChangeDlg* itemDialog1 = this;
+  auto *mainSizer = new wxBoxSizer(wxVERTICAL);
+  SetSizer(mainSizer);
 
-  wxBoxSizer* mainSizer = new wxBoxSizer(wxVERTICAL);
-  mainSizer->AddSpacer(TopMargin);
-  itemDialog1->SetSizer(mainSizer);
-
-  wxStaticText* itemStaticText3 = new wxStaticText( itemDialog1, wxID_STATIC, _("Please enter the current combination, followed by a new combination.\nType the new combination once again to confirm it."), wxDefaultPosition, wxDefaultSize, 0 );
-  mainSizer->Add(itemStaticText3, 0, wxALIGN_LEFT|wxALL, SideMargin);
+  auto *itemStaticText3 = new wxStaticText(
+    this, wxID_STATIC,
+    _("Enter the current master password, followed by a new one.\nType the new one again to confirm it."),
+    wxDefaultPosition, wxDefaultSize, 0
+  );
+  mainSizer->Add(itemStaticText3, 0, wxALL|wxEXPAND, 12);
 
 #ifndef NO_YUBI
-  enum { DLGITEM_COLS = 3 };
-#else
   enum { DLGITEM_COLS = 2 };
+#else
+  enum { DLGITEM_COLS = 1 };
 #endif
 
-  wxFlexGridSizer* itemFlexGridSizer4 = new wxFlexGridSizer(DLGITEM_COLS, 0, 0);
-  itemFlexGridSizer4->AddGrowableCol(1);
-  mainSizer->Add(itemFlexGridSizer4, 1, wxALIGN_LEFT|wxALL|wxEXPAND, SideMargin);
+  auto *itemFlexGridSizer4 = new wxFlexGridSizer(DLGITEM_COLS, 0, 0);
+  itemFlexGridSizer4->AddGrowableCol(0);
+  mainSizer->Add(itemFlexGridSizer4, 1, wxBOTTOM|wxLEFT|wxRIGHT|wxEXPAND, 12);
 
-  wxStaticText* itemStaticText5 = new wxStaticText( itemDialog1, wxID_STATIC, _("Old safe combination:"), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );
-  itemFlexGridSizer4->Add(itemStaticText5, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
+  auto *itemStaticText5 = new wxStaticText(this, wxID_STATIC, _("Old Master Password"), wxDefaultPosition, wxDefaultSize, 0);
+  itemFlexGridSizer4->Add(itemStaticText5, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxBOTTOM, 5);
+#ifndef NO_YUBI
+  itemFlexGridSizer4->AddStretchSpacer(0);
+#endif
 
-  m_oldPasswdEntry = new SafeCombinationCtrl( itemDialog1, ID_OLDPASSWD, &m_oldpasswd, wxDefaultPosition, wxDefaultSize );
+  m_oldPasswdEntry = new SafeCombinationCtrl(this, ID_OLDPASSWD, &m_oldpasswd, wxDefaultPosition, wxDefaultSize);
   m_oldPasswdEntry->SetFocus();
-  itemFlexGridSizer4->Add(m_oldPasswdEntry, 1, wxALIGN_LEFT|wxALL|wxEXPAND, 5);
-
+  itemFlexGridSizer4->Add(m_oldPasswdEntry, 1, wxALIGN_LEFT|wxBOTTOM|wxEXPAND, 12);
 #ifndef NO_YUBI
-  m_YubiBtn = new wxBitmapButton( itemDialog1, ID_YUBIBTN, itemDialog1->GetBitmapResource(wxT("graphics/Yubikey-button.xpm")), wxDefaultPosition, itemDialog1->ConvertDialogToPixels(wxSize(40, 12)), wxBU_AUTODRAW );
-  itemFlexGridSizer4->Add(m_YubiBtn, 0, wxALIGN_CENTER|wxLEFT|wxRIGHT|wxSHAPED, 5);
+  m_YubiBtn = new wxBitmapButton(this, ID_YUBIBTN, GetBitmapResource(wxT("graphics/Yubikey-button.xpm")), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
+  itemFlexGridSizer4->Add(m_YubiBtn, 0, wxLEFT|wxBOTTOM|wxEXPAND, 12);
 #endif
 
-  wxStaticText* itemStaticText8 = new wxStaticText( itemDialog1, wxID_STATIC, _("New safe combination:"), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );
-  itemFlexGridSizer4->Add(itemStaticText8, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-  m_newPasswdEntry = new SafeCombinationCtrl( itemDialog1, ID_NEWPASSWD, &m_newpasswd, wxDefaultPosition, wxDefaultSize );
-  itemFlexGridSizer4->Add(m_newPasswdEntry, 1, wxALIGN_LEFT|wxALL|wxEXPAND, 5);
-
-#ifndef NO_YUBI
-  m_YubiBtn2 = new wxBitmapButton( itemDialog1, ID_YUBIBTN2, itemDialog1->GetBitmapResource(wxT("graphics/Yubikey-button.xpm")), wxDefaultPosition, itemDialog1->ConvertDialogToPixels(wxSize(40, 12)), wxBU_AUTODRAW );
-  itemFlexGridSizer4->Add(m_YubiBtn2, 0, wxALIGN_CENTER|wxLEFT|wxRIGHT|wxSHAPED, 5);
-#endif
-
-  wxStaticText* itemStaticText11 = new wxStaticText( itemDialog1, wxID_STATIC, _("Confirmation:"), wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );
-  itemFlexGridSizer4->Add(itemStaticText11, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);
-
-  m_confirmEntry = new SafeCombinationCtrl( itemDialog1, ID_CONFIRM, &m_confirm, wxDefaultPosition, wxDefaultSize );
-  itemFlexGridSizer4->Add(m_confirmEntry, 1, wxALIGN_LEFT|wxALL|wxEXPAND, 5);
-
+  auto *itemStaticText8 = new wxStaticText(this, wxID_STATIC, _("New Master Password"), wxDefaultPosition, wxDefaultSize, 0);
+  itemFlexGridSizer4->Add(itemStaticText8, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxBOTTOM, 5);
 #ifndef NO_YUBI
   itemFlexGridSizer4->AddStretchSpacer(0);
 #endif
 
-  itemFlexGridSizer4->AddStretchSpacer(0);
-  auto showCombinationCheckBox = new wxCheckBox(this, ID_SHOWCOMBINATION, _("Show Combination"), wxDefaultPosition, wxDefaultSize, 0 );
-  showCombinationCheckBox->SetValue(false);
-  m_isPasswordHidden = true;
-  showCombinationCheckBox->Bind(wxEVT_CHECKBOX, [&](wxCommandEvent& event) {
-    m_oldPasswdEntry->SecureTextfield(!event.IsChecked());
-    m_newPasswdEntry->SecureTextfield(!event.IsChecked());
-    m_confirmEntry->DisableAndClear(event.IsChecked());
-    m_isPasswordHidden = !event.IsChecked();
-    if(event.IsChecked())
-      m_confirmEntry->AllowEmptyCombinationOnce();
-    else
-      m_confirmEntry->ForbidEmptyCombinationOnce();
-  });
-  itemFlexGridSizer4->Add(showCombinationCheckBox, 1, wxALIGN_LEFT|wxALL|wxEXPAND, 5);
-
+  m_newPasswdEntry = new SafeCombinationCtrl(this, ID_NEWPASSWD, &m_newpasswd, wxDefaultPosition, wxDefaultSize);
+  itemFlexGridSizer4->Add(m_newPasswdEntry, 1, wxALIGN_LEFT|wxBOTTOM|wxEXPAND, 12);
 #ifndef NO_YUBI
-  itemFlexGridSizer4->AddStretchSpacer(0);
-
-  m_yubiStatusCtrl = new wxStaticText( itemDialog1, ID_YUBISTATUS, _("Please insert your YubiKey"), wxDefaultPosition, wxDefaultSize, 0 );
-  mainSizer->Add(m_yubiStatusCtrl, 0, wxEXPAND|wxALL, SideMargin);
+  m_YubiBtn2 = new wxBitmapButton(this, ID_YUBIBTN2, GetBitmapResource(wxT("graphics/Yubikey-button.xpm")), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT);
+  itemFlexGridSizer4->Add(m_YubiBtn2, 0, wxLEFT|wxBOTTOM|wxEXPAND, 12);
 #endif
 
-  mainSizer->AddSpacer(RowSeparation);
-  mainSizer->Add(new wxStaticLine(this), 0, wxLEFT|wxRIGHT|wxEXPAND, SideMargin);
-  mainSizer->AddSpacer(RowSeparation);
+  auto *itemStaticText11 = new wxStaticText(this, wxID_STATIC, _("Confirmation"), wxDefaultPosition, wxDefaultSize, 0);
+  itemFlexGridSizer4->Add(itemStaticText11, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxBOTTOM, 5);
+#ifndef NO_YUBI
+  itemFlexGridSizer4->AddStretchSpacer(0);
+#endif
 
-  wxStdDialogButtonSizer* itemStdDialogButtonSizer15 = new wxStdDialogButtonSizer;
+  m_confirmEntry = new SafeCombinationCtrl(this, ID_CONFIRM, &m_confirm, wxDefaultPosition, wxDefaultSize);
+  itemFlexGridSizer4->Add(m_confirmEntry, 1, wxALIGN_LEFT|wxBOTTOM|wxEXPAND, 12);
+#ifndef NO_YUBI
+  itemFlexGridSizer4->AddStretchSpacer(0);
+#endif
 
-  mainSizer->Add(itemStdDialogButtonSizer15, 0, wxLEFT|wxRIGHT|wxEXPAND, SideMargin);
-  wxButton* itemButton16 = new wxButton( itemDialog1, wxID_OK, _("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemButton16->SetDefault();
-  itemStdDialogButtonSizer15->AddButton(itemButton16);
+#ifndef NO_YUBI
+  auto* panel = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxSize(-1,  40));
+  mainSizer->Add(panel, 0, wxLEFT|wxRIGHT|wxEXPAND|wxBOTTOM, 12);
 
-  wxButton* itemButton17 = new wxButton( itemDialog1, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemStdDialogButtonSizer15->AddButton(itemButton17);
+  auto* horizontalBoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
+  panel->SetSizer(horizontalBoxSizer1);
 
-  wxButton* itemButton18 = new wxButton( itemDialog1, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, 0 );
-  itemStdDialogButtonSizer15->AddButton(itemButton18);
+  m_yubiStatusCtrl = new wxStaticText(panel, ID_YUBISTATUS, _("Insert your YubiKey"), wxDefaultPosition, wxDefaultSize, 0);
+  horizontalBoxSizer1->Add(m_yubiStatusCtrl, 0, wxALIGN_LEFT|wxEXPAND, 0);
+#endif
 
-  itemStdDialogButtonSizer15->Realize();
+  auto horizontalBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
+  mainSizer->Add(horizontalBoxSizer3, 0, wxEXPAND|wxLEFT|wxBOTTOM|wxRIGHT, 12);
 
-  mainSizer->AddSpacer(BottomMargin);
+  horizontalBoxSizer3->AddSpacer(90);
+  horizontalBoxSizer3->AddStretchSpacer();
 
-////@end SafeCombinationChangeDlg content construction
+  horizontalBoxSizer3->Add(
+    new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, 0),
+    0, wxALIGN_CENTER_VERTICAL|wxALL, 5
+  );
+
+  auto *changeButton = new wxButton(this, wxID_OK, _("&Change"), wxDefaultPosition, wxDefaultSize, 0);
+  horizontalBoxSizer3->Add(
+    changeButton,
+    0, wxALIGN_CENTER_VERTICAL|wxALL, 5
+  );
+  changeButton->SetDefault();
+
+  horizontalBoxSizer3->AddSpacer(60);
+  horizontalBoxSizer3->AddStretchSpacer();
+
+  if (wxUtilities::IsVirtualKeyboardSupported()) {
+    auto *keyboardButton = new ExternalKeyboardButton(this);
+    keyboardButton->SetFocusOnSafeCombinationCtrl(m_oldPasswdEntry);
+    horizontalBoxSizer3->Add(
+      keyboardButton,
+      0, wxALIGN_CENTER_VERTICAL|wxALL, 0
+    );
+  }
 }
 
 /*!
@@ -273,43 +265,25 @@ void SafeCombinationChangeDlg::OnOkClick(wxCommandEvent& WXUNUSED(evt))
     const StringX old = m_oldresponse.empty() ? m_oldpasswd : m_oldresponse;
     int rc = m_core.CheckPasskey(m_core.GetCurFile(), old);
     if (rc == PWScore::WRONG_PASSWORD) {
-      wxMessageDialog err(this, _("The old safe combination is not correct"),
+      wxMessageDialog err(this, _("The old master password is not correct"),
                           _("Error"), wxOK | wxICON_EXCLAMATION);
       err.ShowModal();
     } else if (rc == PWScore::CANT_OPEN_FILE) {
-      wxMessageDialog err(this, _("Cannot verify old safe combination - file gone?"),
+      wxMessageDialog err(this, _("Cannot verify old master password - file gone?"),
                           _("Error"), wxOK | wxICON_EXCLAMATION);
       err.ShowModal();
-    } else if (m_isPasswordHidden && (m_confirm != m_newpasswd)) {
-      wxMessageDialog err(this, _("New safe combination and confirmation do not match"),
+    } else if (m_confirm != m_newpasswd) {
+      wxMessageDialog err(this, _("New master password and confirmation do not match"),
                           _("Error"), wxOK | wxICON_EXCLAMATION);
       err.ShowModal();
-    // Vox populi vox dei - folks want the ability to use a weak
-    // passphrase, best we can do is warn them...
-    // If someone want to build a version that insists on proper
-    // passphrases, then just define the preprocessor macro
-    // PWS_FORCE_STRONG_PASSPHRASE in the build properties/Makefile
-    // (also used in CPasskeySetup)
-    } else if (!CPasswordCharPool::CheckMasterPassword(m_newpasswd, errmess)) {
-      wxString msg = _("Weak passphrase:");
-      msg += wxT("\n\n");
-      msg += errmess.c_str();
-#ifndef PWS_FORCE_STRONG_PASSPHRASE
-      msg += wxT("\n");
-      msg += _("Use it anyway?");
-      wxMessageDialog err(this, msg,
-                          _("Error"), wxYES_NO | wxICON_HAND);
-      int rc1 = err.ShowModal();
-      if (rc1 == wxID_YES)
-        EndModal(wxID_OK);
-#else
-      wxMessageDialog err(this, msg,
-                          _("Error"), wxOK | wxICON_HAND);
-      err.ShowModal();
-#endif // PWS_FORCE_STRONG_PASSPHRASE
-    } else { // password checks out OK.
-      EndModal(wxID_OK);
+    } else if (CheckPasswordStrengthAndWarn(this, m_newpasswd)) {
+      EndModal(wxID_OK); // password checks out OK.
     }
+    // If we got here, there was an error in a PW entry.  A case exists where switching from a
+    // (Yubikey without a PW), to (PW only), we may need to allow the old PW to be
+    // blank again for the next retry.
+    if (!m_oldresponse.empty() && m_oldpasswd.empty())
+      m_oldPasswdEntry->AllowEmptyCombinationOnce();
   }
 }
 
@@ -349,10 +323,16 @@ void SafeCombinationChangeDlg::OnYubibtnClick(wxCommandEvent& WXUNUSED(event))
       if (rc == PWScore::WRONG_PASSWORD) {
         m_oldresponse.clear();
         m_yubiStatusCtrl->SetForegroundColour(*wxRED);
-        m_yubiStatusCtrl->SetLabel(_("YubiKey safe combination incorrect"));
+        m_yubiStatusCtrl->SetLabel(_("YubiKey master password incorrect"));
       } else {
-        m_yubiMixin2.SetPrompt1(_("Enter new safe combination (if any) and click on bottom Yubikey button"));
+        // The old password has been validated.  If it was blank, allow it to be blank on the next check.
+        // e.g. changing from (Yubikey w/blank PW) to (PW only).
+        if (m_oldpasswd.empty()) {
+          m_oldPasswdEntry->AllowEmptyCombinationOnce();
+        }
+        m_yubiMixin2.SetPrompt1(_("Enter new master password and click on bottom Yubikey button"));
         m_yubiMixin2.UpdateStatus();
+        m_newPasswdEntry->SetFocus();
       }
     }
   }
@@ -374,7 +354,7 @@ void SafeCombinationChangeDlg::OnYubibtn2Click(wxCommandEvent& WXUNUSED(event))
     // 1. Both old password and old response can't be blank
     if (m_oldpasswd.empty() && m_oldresponse.empty()) {
       m_yubiStatusCtrl->SetForegroundColour(*wxRED);
-      m_yubiStatusCtrl->SetLabel(_("Please confirm existing combination"));
+      m_yubiStatusCtrl->SetLabel(_("Confirm existing master password"));
       return;
     }
     // 2. If there's an old response, it should already have been checked, but JIC:
@@ -383,7 +363,7 @@ void SafeCombinationChangeDlg::OnYubibtn2Click(wxCommandEvent& WXUNUSED(event))
       if (rc == PWScore::WRONG_PASSWORD) {
         m_oldresponse.clear();
         m_yubiStatusCtrl->SetForegroundColour(*wxRED);
-        m_yubiStatusCtrl->SetLabel(_("YubiKey safe combination incorrect"));
+        m_yubiStatusCtrl->SetLabel(_("YubiKey master password incorrect"));
         return;
       }
     } else {
@@ -391,10 +371,22 @@ void SafeCombinationChangeDlg::OnYubibtn2Click(wxCommandEvent& WXUNUSED(event))
       rc = m_core.CheckPasskey(m_core.GetCurFile(), m_oldpasswd);
       if (rc == PWScore::WRONG_PASSWORD) {
         m_yubiStatusCtrl->SetForegroundColour(*wxRED);
-        m_yubiStatusCtrl->SetLabel(_("Current safe combination incorrect"));
+        m_yubiStatusCtrl->SetLabel(_("Current master password incorrect"));
         return;
       }
     }
+
+    if (m_confirm != m_newpasswd) {
+      wxMessageDialog err(this, _("New master password and confirmation do not match"),
+                          _("Error"), wxOK | wxICON_EXCLAMATION);
+      err.ShowModal();
+      return;
+    }
+    // A blank password with a Yubikey is a common use case
+    if (!m_newpasswd.empty() && !CheckPasswordStrengthAndWarn(this, m_newpasswd)) {
+      return;
+    }
+
     StringX response;
     bool oldYubiChallenge = ::wxGetKeyState(WXK_SHIFT); // for pre-0.94 databases
     if (m_yubiMixin2.PerformChallengeResponse(this, m_newpasswd, response, oldYubiChallenge)) {
