@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2023 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2025 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -122,15 +122,15 @@ wxMenu* SystemTray::CreatePopupMenu()
 
   switch (m_status) {
     case TrayStatus::UNLOCKED:
-        menu->Append(ID_SYSTRAY_LOCK, _("&Lock Safe"))->SetBitmap(wxBitmap(lock_xpm));
+        menu->Append(ID_SYSTRAY_LOCK, _("&Lock"))->SetBitmap(wxBitmap(lock_xpm));
       break;
 
     case TrayStatus::LOCKED:
-        menu->Append(ID_SYSTRAY_UNLOCK, _("&Unlock Safe"))->SetBitmap(wxBitmap(unlock_xpm));
+        menu->Append(ID_SYSTRAY_UNLOCK, _("&Unlock"))->SetBitmap(wxBitmap(unlock_xpm));
         break;
 
     case TrayStatus::CLOSED:
-        menu->Append(wxID_NONE, _("No Safe Open"));
+        menu->Append(wxID_NONE, _("No Password Database Open"));
         break;
 
     default:
@@ -311,7 +311,15 @@ void SystemTray::ProcessSysTrayMenuItem(int itemId)
       break;
 
     case ID_SYSTRAY_UNLOCK:
+#ifdef __WXOSX__
+      // The intent of this menu option is to unlock the DB without restoring the window.
+      // On macOS, an empty window frame appears with no UI elements.
+      // I couldn't find a way to suppress the window frame, so the workaround is to
+      // go ahead and fully restore the UI.  If a better way is found, this ifdef can be removed.
+      m_frame->UnlockSafe(true, false); // true => restore UI
+#else
       m_frame->UnlockSafe(false, false); // false => don't restore UI
+#endif
       break;
 
     case ID_SYSTRAY_CLEAR_RUE:
